#include <Wire.h>
#include <Adafruit_ADS1015.h>

// сопротивление термистора при 25 градусах по Цельсию. Термисторы минимум 5% точности.
#define THERMISTORNOMINAL 10000

// temp. для номинального сопротивления (практически всегда равна 25 C)
#define TEMPERATURENOMINAL 25

// бета коэффициент термистора из даташита (обычно 3000-4000)
#define BCOEFFICIENT 3950


// сопротивление верхнего (подключенного к опорному напряжению) резистора. Для середины нашего рабочего диапазона температур
// получается 1,5 кОм, желательно 1% точности
#define SERIESRESISTOR 1500

// количество отсчетов для усреднения после фильтрации
#define numsamples 64

float sample_0;
float sample_1;
float sample_2;
float sample_3;

float buf0[1];
float buf1[1];
float buf2[1];
float buf3[1];

float steinhart_0;
float steinhart_1;
float steinhart_2;
float steinhart_3;

float T_1;
float T_2;
float T_3;
float T_4;

uint8_t i;

// Фильтры НЧ 1 порядка (1000 / 1 частота выборок / среза)для каналов АЦП
float process0(register float val) {
    
    register float tmp, fir, iir;
    tmp = buf0[0];
    iir = val * 0.003131764229192702f;
    iir -= -0.9937364715416146f * tmp; 
    fir = tmp;
    fir += iir;
    buf0[0] = iir; 
    val = fir;
    return val;
  }

// Фильтры НЧ 1 порядка (1000 / 1 частота выборок / среза)для каналов АЦП
float process1(register float val) {
    
    register float tmp, fir, iir;
    tmp = buf1[0];
    iir = val * 0.003131764229192702f;
    iir -= -0.9937364715416146f * tmp; 
    fir = tmp;
    fir += iir;
    buf1[0] = iir; 
    val = fir;
    return val;
  }

// Фильтры НЧ 1 порядка (1000 / 1 частота выборок / среза)для каналов АЦП
float process2(register float val) {
    
    register float tmp, fir, iir;
    tmp = buf2[0];
    iir = val * 0.003131764229192702f;
    iir -= -0.9937364715416146f * tmp; 
    fir = tmp;
    fir += iir;
    buf2[0] = iir; 
    val = fir;
    return val;
  }

// Фильтры НЧ 1 порядка (1000 / 1 частота выборок / среза)для каналов АЦП
float process3(register float val) {
    
    register float tmp, fir, iir;
    tmp = buf3[0];
    iir = val * 0.003131764229192702f;
    iir -= -0.9937364715416146f * tmp; 
    fir = tmp;
    fir += iir;
    buf3[0] = iir; 
    val = fir;
    return val;
  }

Adafruit_ADS1115 ads(0x48);  /* Адрес ADDR_To_GND */

void setup(void)
{

  Serial.begin(9600);

  // The ADC input range (or gain) can be changed via the following
  // functions, but be careful never to exceed VDD +0.3V max, or to
  // exceed the upper and lower limits if you adjust the input range!
  // Setting these values incorrectly may destroy your ADC!
  //                                                                ADS1015  ADS1115
  //                                                                -------  -------
  // ads.setGain(GAIN_TWOTHIRDS);  // 2/3x gain +/- 6.144V  1 bit = 3mV      0.1875mV (default)
  ads.setGain(GAIN_ONE);        // 1x gain   +/- 4.096V  1 bit = 2mV      0.125mV
  // ads.setGain(GAIN_TWO);        // 2x gain   +/- 2.048V  1 bit = 1mV      0.0625mV
  // ads.setGain(GAIN_FOUR);       // 4x gain   +/- 1.024V  1 bit = 0.5mV    0.03125mV
  // ads.setGain(GAIN_EIGHT);      // 8x gain   +/- 0.512V  1 bit = 0.25mV   0.015625mV
  // ads.setGain(GAIN_SIXTEEN);    // 16x gain  +/- 0.256V  1 bit = 0.125mV  0.0078125mV

  ads.begin();
}

void loop(void)
{

  for (i = 0; i < numsamples; i++) { //вычитываем значения из АЦП

    sample_0 = ads.readADC_SingleEnded(0);
    //delay(10);
    sample_1 = ads.readADC_SingleEnded(1);
    //delay(10);
    sample_2 = ads.readADC_SingleEnded(2);
    //delay(10);
    sample_3 = ads.readADC_SingleEnded(3);
    //delay(10);

    sample_0 = (sample_0 * 0.125) / 1000; //пересчитываем в привычные вольты
    sample_1 = (sample_1 * 0.125) / 1000;
    sample_2 = (sample_2 * 0.125) / 1000;
    sample_3 = (sample_3 * 0.125) / 1000;

    sample_0 = 3.305 / sample_0 - 1; // конвертируем значение в сопротивление
    sample_0 = SERIESRESISTOR / sample_0;
    sample_1 = 3.305 / sample_1 - 1;
    sample_1 = SERIESRESISTOR / sample_1;
    sample_2 = 3.305 / sample_2 - 1;
    sample_2 = SERIESRESISTOR / sample_2;
    sample_3 = 3.305 / sample_3 - 1;
    sample_3 = SERIESRESISTOR / sample_3;

    // конвертируем сопротивление в температуру по Стейнхарту-Харту
    steinhart_0 = sample_0 / THERMISTORNOMINAL; // (R/Ro)
    steinhart_1 = sample_1 / THERMISTORNOMINAL;
    steinhart_2 = sample_2 / THERMISTORNOMINAL;
    steinhart_3 = sample_3 / THERMISTORNOMINAL;

    steinhart_0 = log(steinhart_0); // ln(R/Ro)
    steinhart_1 = log(steinhart_1);
    steinhart_2 = log(steinhart_2);
    steinhart_3 = log(steinhart_3);

    steinhart_0 /= BCOEFFICIENT; // 1/B * ln(R/Ro)
    steinhart_1 /= BCOEFFICIENT;
    steinhart_2 /= BCOEFFICIENT;
    steinhart_3 /= BCOEFFICIENT;

    steinhart_0 += 1.0 / (TEMPERATURENOMINAL + 273.15); // + (1/To)
    steinhart_1 += 1.0 / (TEMPERATURENOMINAL + 273.15);
    steinhart_2 += 1.0 / (TEMPERATURENOMINAL + 273.15);
    steinhart_3 += 1.0 / (TEMPERATURENOMINAL + 273.15);

    steinhart_0 = 1.0 / steinhart_0; // инвертируем
    steinhart_1 = 1.0 / steinhart_1;
    steinhart_2 = 1.0 / steinhart_2;
    steinhart_3 = 1.0 / steinhart_3;

    steinhart_0 -= 273.15; // конвертируем в градусы по Цельсию 
    steinhart_1 -= 273.15; 
    steinhart_2 -= 273.15; 
    steinhart_3 -= 273.15;

    steinhart_0 = process0(steinhart_0); //фильтруем
    steinhart_1 = process1(steinhart_1);
    steinhart_2 = process2(steinhart_2);
    steinhart_3 = process3(steinhart_3);


    T_1 += steinhart_0; //суммируем температуры
    T_2 += steinhart_1;
    T_3 += steinhart_2;
    T_4 += steinhart_3;

  }

  // рассчитываем среднее значение температуры и корректируем
  T_1 = T_1 /= numsamples; 
  T_1 += 0.1623;
  T_2 = T_2 /= numsamples; 
  T_2 += 0.057;
  T_3 = T_3 /= numsamples; 
  T_3 += 0.3701;
  T_4 = T_4 /= numsamples; 

  float delta; //Считаем дельту
  delta = T_3 - T_4;

  Serial.print("{00<"); //выводим токен начала отчета, адрес первого термометра, токен типа
  Serial.print(T_1, 4); //Выводим температуру
  Serial.print(" 01<");
  Serial.print(T_2, 4);
  Serial.print(" 02<");
  Serial.print(T_3, 4);
  Serial.print(" 03<");
  Serial.print(T_4, 4);
  Serial.print(" 04<");
  Serial.print(delta, 4);
  Serial.println(" }");

}

